# -*- coding: utf-8 -*-

# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

TRAITS = [
    # Traits corresponding to the allowed values of "hw_usb_model"
    # image metadata property. These are used by the nova libvirt driver
    # however we have decided for now to not support the other older options
    # for performance and support reasons. Options are documented at
    # https://libvirt.org/formatdomain.html#controllers
    'MODEL_QEMU_XHCI',
    'MODEL_NEC_XHCI'
]
